/* Author : Dario GENGA
 * Date : 14.12.2021
 * Description : The power 4 game
 */

#include <stdio.h>
#include <stdlib.h>
#include "puissance.h"

void print_help() {
    printf("Usage: puissance4 <mode> <row> <col>\n");
    printf("    mode  specifies the mode:\n");
    printf("        1 = single player game (random),\n");
    printf("        2 = single player game (AI),\n");
    printf("        3 = two players game\n");
    printf("    row   specifies the number of rows (>= 4)\n");
    printf("    col   specifies the number of columns (>= 4)\n");
}

void play_game(puissance game) {
    int selected_col_index = -1;

    while(game.state == ONGOING) {
        // First player action (always a human)
        if (game.current_player == PLAYER_ONE || game.mode == TWO_PLAYERS) {
            printf("\nColumn number? (starts at 1):");
            do {
                scanf("%d", &selected_col_index);
                selected_col_index -= 1;
            } while (manual_play(&game, selected_col_index) == false);
        }
        // Second player action (can be the computer)
        else {
            if (game.mode == RAND_AI) {
                random_play(&game);
            } else {
                smart_play(&game);
            };
        }
        print_game(game);
    }
}

int main(int argc, char *argv[]) {
    srand(0);
    puissance game;

    // Get game arguments
    if (argc != 4) {
        print_help();
        return EXIT_FAILURE;
    }
    GameMode mode = atoi(argv[1]) - 1;
    int rows = atoi(argv[2]);
    int cols = atoi(argv[3]);

    // Initialize the game
    game_init(&game, mode, rows, cols);
    printf("Board size is %dx%d (rows x col)", rows, cols);
    print_game(game);

    // Start playing
    play_game(game);

    // Free the memory and exit the program
    game_destroy(&game);
    return EXIT_SUCCESS;
}
