/* Author : Dario GENGA
 * Date : 14.12.2021
 * Description : The power 4 game
 */

#ifndef _PUISSANCE_H
#define _PUISSANCE_H
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#define ROW_MIN 4
#define COL_MIN 4
#define DEFAULT_ROW 6
#define DEFAULT_COL 7
#define PLAYER_ONE_STRING "X"
#define PLAYER_TWO_STRING "O"
#define EMPTY_CELL_VALUE -2
#define NB_SAME_VALUE_ALIGNED_FOR_WIN 4
#define NB_VERIFICATION_FOR_WIN (NB_SAME_VALUE_ALIGNED_FOR_WIN - 1)

typedef enum {
    RAND_AI,
    SMART_AI,
    TWO_PLAYERS,
} GameMode;

typedef enum {
    PLAYER_ONE,
    PLAYER_TWO,
} Player;

typedef enum {
    ONGOING,
    DRAW,
    PLAYER_ONE_WIN,
    PLAYER_TWO_WIN,
} GameResult;

typedef struct _puissance {
    GameMode mode;
    GameResult state;
    Player current_player;
    int row;
    int col;
    int** data;
} puissance;

void game_init(puissance *p, GameMode mode, int row, int col);
puissance game_copy(puissance *p);
void print_game(puissance p);
GameResult verify_game(puissance *p, int last_col_index_played);
GameResult display_game_result(puissance p);
int get_available_row_index(puissance *p, int selected_col_index);
bool manual_play(puissance *p, int selected_col_index);
bool random_play(puissance *p);
bool smart_play(puissance *p);
void game_destroy(puissance *p);

// TODO: add get top of col

#endif